'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _exportPlugin = require('../../../lib/exportPlugin');

var _exportPlugin2 = _interopRequireDefault(_exportPlugin);

var _plugin = require('../../../lib/plugin');

var _plugin2 = _interopRequireDefault(_plugin);

var _package = require('../package.json');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var VimeoAnalytics = function (_VideoPlugin) {
  _inherits(VimeoAnalytics, _VideoPlugin);

  function VimeoAnalytics(player, authToken) {
    _classCallCheck(this, VimeoAnalytics);

    var _this = _possibleConstructorReturn(this, (VimeoAnalytics.__proto__ || Object.getPrototypeOf(VimeoAnalytics)).call(this, _package.name, _package.version));

    _this.authToken = authToken;
    _this.player = player;
    _this.metadata = {
      content: {},
      playback: { videoPlayer: 'Vimeo' }
    };
    _this.mostRecentHeartbeat = 0;
    _this.isPaused = false;
    return _this;
  }

  _createClass(VimeoAnalytics, [{
    key: 'initialize',
    value: function initialize() {
      var events = {
        loaded: this.retrieveMetadata,
        play: this.trackPlay,
        pause: this.trackPause,
        ended: this.trackEnded,
        timeupdate: this.trackHeartbeat
      };

      for (event in events) {
        this.registerHandler(event, events[event]);
      }
    }

    // This is a helper function used to facilitate easier testing.

  }, {
    key: 'registerHandler',
    value: function registerHandler(event, handler) {
      var _this2 = this;

      this.player.on(event, function (data) {
        _this2.updateMetadata(data);
        handler.call(_this2, data);
      });
    }
  }, {
    key: 'trackPlay',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (this.isPaused) {
                  this.track('Video Playback Resumed', this.metadata.playback);
                  this.isPaused = false;
                } else {
                  this.track('Video Playback Started', this.metadata.playback);
                  this.track('Video Content Started', this.metadata.content);
                }

              case 1:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function trackPlay() {
        return _ref.apply(this, arguments);
      }

      return trackPlay;
    }()
  }, {
    key: 'trackEnded',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                this.track('Video Playback Completed', this.metadata.playback);
                this.track('Video Content Completed', this.metadata.content);

              case 2:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function trackEnded() {
        return _ref2.apply(this, arguments);
      }

      return trackEnded;
    }()

    // Vimeo provides time updates every 250ms while Segment documents sending heartbeats every 10s.
    // Therefore, we need to do some math to ensure we are not sending 4 heartbeat events when we reach
    // 10 second intervals.

  }, {
    key: 'trackHeartbeat',
    value: function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
        var mostRecentHeartbeat, currentPosition;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                mostRecentHeartbeat = this.mostRecentHeartbeat;
                currentPosition = this.metadata.playback.position;

                if (currentPosition !== mostRecentHeartbeat && currentPosition - mostRecentHeartbeat >= 10) {
                  this.track('Video Content Playing', this.metadata.content);
                  this.mostRecentHeartbeat = Math.floor(currentPosition);
                }

              case 3:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function trackHeartbeat() {
        return _ref3.apply(this, arguments);
      }

      return trackHeartbeat;
    }()
  }, {
    key: 'trackPause',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                this.isPaused = true;
                this.track('Video Playback Paused', this.metadata.playback);

              case 2:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function trackPause() {
        return _ref4.apply(this, arguments);
      }

      return trackPause;
    }()

    // retrieve static video metadata from vimeo API

  }, {
    key: 'retrieveMetadata',
    value: function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(data) {
        var videoId, res, err, json;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                videoId = data.id;
                _context5.next = 3;
                return fetch('https://api.vimeo.com/videos/' + videoId, {
                  headers: {
                    Authorization: 'Bearer ' + this.authToken
                  }
                });

              case 3:
                res = _context5.sent;

                if (res.ok) {
                  _context5.next = 10;
                  break;
                }

                _context5.next = 7;
                return res.json();

              case 7:
                err = _context5.sent;

                console.error('Request to Vimeo API failed with: ', err);
                return _context5.abrupt('return');

              case 10:
                _context5.next = 12;
                return res.json();

              case 12:
                json = _context5.sent;


                this.metadata.content.title = json.name;
                this.metadata.content.description = json.description;
                this.metadata.content.publisher = json.user.name;

                this.metadata.playback.position = 0;
                this.metadata.playback.totalLength = json.duration;

              case 18:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function retrieveMetadata(_x) {
        return _ref5.apply(this, arguments);
      }

      return retrieveMetadata;
    }()

    // update dynamic video information

  }, {
    key: 'updateMetadata',
    value: function () {
      var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(data) {
        var volume;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return this.player.getVolume();

              case 2:
                volume = _context6.sent;

                if (volume) this.metadata.playback.sound = volume * 100;
                this.metadata.playback.position = data.seconds;

              case 5:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function updateMetadata(_x2) {
        return _ref6.apply(this, arguments);
      }

      return updateMetadata;
    }()
  }]);

  return VimeoAnalytics;
}(_plugin2.default);

exports.default = (0, _exportPlugin2.default)(VimeoAnalytics, 'VimeoAnalytics');
